#include <windows.h>

#include "ifinfo.h"

typedef struct _MIB_IPADDRROW {
  DWORD   dwAddr;              // IP address
  DWORD   dwIndex;             // interface index
  DWORD   dwMask;              // subnet mask
  DWORD   dwBCastAddr;         // broadcast address 
  DWORD   dwReasmSize;         // rassembly size 
  unsigned short   unused1;    // not currently used 
  unsigned short   unused2;    // not currently used 
} MIB_IPADDRROW, *PMIB_IPADDRROW;

typedef struct _MIB_IPADDRTABLE {
  DWORD         dwNumEntries;    // number of entries in the table
  MIB_IPADDRROW table[1]; // array of IP address entries
} MIB_IPADDRTABLE, *PMIB_IPADDRTABLE;


// returns TRUE if network list changed
int getNetworkAdapterList(ifInfo *addrlist, int size_addrlist)
{
  static HMODULE hMod;
  static DWORD (WINAPI *_GetIpAddrTable)(PMIB_IPADDRTABLE pIpAddrTable, PULONG pdwSize, BOOL bOrder);

  if (!hMod) hMod = LoadLibrary("iphlpapi.dll");
  if (!hMod) return FALSE;

  if (!_GetIpAddrTable) *(void **)&_GetIpAddrTable = (void *)GetProcAddress(hMod,"GetIpAddrTable");
  if (!_GetIpAddrTable) return FALSE;

  char _buf[128*1024];
  DWORD bufl=sizeof(_buf);
  MIB_IPADDRTABLE *addrt = (MIB_IPADDRTABLE *)&_buf;
  int changed_stat=0;
  int addrlist_pos=0;

  if (_GetIpAddrTable(addrt,&bufl,TRUE) == NO_ERROR)
  {
    DWORD x;     
    for (x = 0; x < addrt->dwNumEntries; x ++)
    {
      if (!addrt->table[x].dwAddr) continue;

      // ignore 127.0.0.* and 169.254.*
      if ((addrt->table[x].dwAddr & 0x00ffffff) == 0x00007f || 
          (addrt->table[x].dwAddr & 0x0000ffff) == 0xFEA9
            ) continue;

      unsigned int b=ntohl(addrt->table[x].dwMask);
      char maskbits=32;
      while (!(b&0x1) && maskbits > 0 )
      {
        maskbits--;
        b>>=1;
      }

      maskbits=max(maskbits,24); // this is to avoid doing /16's etc
     
      if (addrlist[addrlist_pos].ip != addrt->table[x].dwAddr || maskbits != addrlist[addrlist_pos].maskbits) 
      {
        changed_stat++;
        addrlist[addrlist_pos].ip=addrt->table[x].dwAddr;
        addrlist[addrlist_pos].maskbits=maskbits;
      }
      addrlist_pos++;
    }
    for (; addrlist_pos < size_addrlist; addrlist_pos ++)
    {
      if (addrlist[addrlist_pos].ip) changed_stat++;
      addrlist[addrlist_pos].ip=0;
      addrlist[addrlist_pos].maskbits=0;
    }
  }

  return changed_stat;

}
