/*
    LanMon
    Copyright (C) 2004-2005 Cockos Incorporated.

    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.
      
*/


#include <windows.h>
#include <commctrl.h>
#include "../wdl/string.h"
#include "../wdl/ptrlist.h"
#include "../wdl/jnetlib/jnetlib.h"
#include "../wdl/wingui/wndsize.h"

#include "resource.h"

#include "ifinfo.h"

#include "ping.h"

#include "portscan.h"

static BOOL CALLBACK MainDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

DWORD g_config_netrefresh=30000;
int g_config_pingtimeout=300;
extern int g_config_max_simul_pings;



HWND g_hwndDlg;

DWORD g_last_netrefreshtime;
DWORD g_last_scan_ip;

ifInfo g_networks[64];


WDL_PtrList<MonitoredHost> g_hostlist;
static int HostSearchCompare(const void *elem1, const void *elem2)
{
  MonitoredHost *a = *(MonitoredHost **)elem1;
  MonitoredHost *b = *(MonitoredHost **)elem2;
  return a->m_address - b->m_address;

}
HINSTANCE g_hInstance;

static MonitoredHost *m_scanning_host;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  g_hInstance=hInstance;
  InitCommonControls();

  WSADATA wsaData;
  WSAStartup(MAKEWORD(2, 2), &wsaData);

  ping_init();
  portscan_init();

  DialogBox(hInstance,MAKEINTRESOURCE(IDD_MAIN),NULL,MainDlgProc);

  portscan_quit();
  ping_quit();

  return 0;
}

int g_net_scanif,g_net_scanidx;


HWND g_listview;

BOOL CALLBACK MainDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static WDL_WndSizer resizer;
  switch (uMsg)
  {
    case WM_INITDIALOG:

      SetClassLong(hwndDlg,GCL_HICON,(LONG)LoadIcon(g_hInstance,MAKEINTRESOURCE(IDI_ICON1)));
      resizer.init(hwndDlg);
      resizer.init_item(IDC_NETWORKS,0,0,1,0);
      resizer.init_item(IDC_LIST1,0,0,1,1);
      resizer.init_item(IDC_STATUS,0,1,1,1);


      g_hwndDlg=hwndDlg;
      
      g_listview=GetDlgItem(hwndDlg,IDC_LIST1);
      {
        LVCOLUMN col={LVCF_TEXT|LVCF_WIDTH,0,90,"IP",};
        ListView_InsertColumn(g_listview,0,&col);
        col.pszText="Ping (ms)";
        col.cx=66;
        ListView_InsertColumn(g_listview,1,&col);
        col.pszText="Services";
        col.cx=400;
        ListView_InsertColumn(g_listview,2,&col);
      }
      SetTimer(hwndDlg,100,1000,NULL);
      SetTimer(hwndDlg,500,10,NULL);
      PostMessage(hwndDlg,WM_TIMER,100,0);
    return 0;
    case WM_SIZE:
      if (wParam != SIZE_MINIMIZED) {
        resizer.onResize();
      }
    return 0;
    case WM_GETMINMAXINFO:
      {
        LPMINMAXINFO p=(LPMINMAXINFO)lParam;
        p->ptMinTrackSize.x = 280;
        p->ptMinTrackSize.y = 160;
      }
    return 0;

    case WM_CLOSE:
      EndDialog(hwndDlg,1);
    return 0;
    case WM_NOTIFY:
      {
        LPNMHDR l=(LPNMHDR)lParam;
        if (l->idFrom == IDC_LIST1)
        {
          if (l->code == LVN_GETDISPINFO)
          {
            NMLVDISPINFO *lpdi = (NMLVDISPINFO*) lParam;
            int item=lpdi->item.iItem;

            if (item >= 0 && item < g_hostlist.GetSize())
            {
              MonitoredHost *m =g_hostlist.Get(item);
              if (m)
              {
                if (lpdi->item.mask & LVIF_TEXT)
                {
                  char buf[256];
                  buf[0]=0;
                  switch (lpdi->item.iSubItem)
                  {
                    case 0:
                      {
                        int ip=m->m_address;
                        wsprintf(buf,"%d.%d.%d.%d",((ip>>24)&0xff),(ip>>16)&0xff,(ip>>8)&0xff,(ip)&0xff);
                      }
                    break;
                    case 1:
                      if (m->m_last_ping < 10)
                        strcpy(buf,"<10");
                      else wsprintf(buf,"%d",m->m_last_ping);
                    break;
                    case 2:
                      {
                        WDL_String str;
                        if (m->m_has_scanned)
                        {
                          if (m->m_has_httpd) 
                          {
                            str.Append("HTTP: ");
                            str.Append(m->m_has_httpd);
                          }
                          if (m->m_has_netbiosssn) 
                          {
                            if (str.Get()[0]) str.Append(",");
                            str.Append("NETBIOS");
                          }
                        }
                        else str.Set("?");

                        lstrcpyn(buf,str.Get(),sizeof(buf));
                      }
                    break;
                  }
                  lstrcpyn(lpdi->item.pszText,buf,lpdi->item.cchTextMax);
                }
              }
            }

          }

        }
      }
    return 0;
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK: 
        case IDCANCEL: return 0;


      }
    return 0;
    case WM_TIMER:
      if (wParam == 500)
      {
        if (!g_networks[0].ip) break;


        if (m_scanning_host && run_portscan(m_scanning_host))
        {
          MonitoredHost **res;

          res = (MonitoredHost **)bsearch((void *) &m_scanning_host,(void *)g_hostlist.GetList(),g_hostlist.GetSize(),sizeof(void *),HostSearchCompare);
          // redraw m_scanning_host's items
          if (res)
          {
            int pos=res - g_hostlist.GetList();
            ListView_RedrawItems(g_listview,pos,pos);
          }

          m_scanning_host=0;
        }
    
        if (ping_avail()>0)
        {
          // wrap ip
          if (g_net_scanidx >= (1<<(32-g_networks[g_net_scanif].maskbits)) - 2) // don't ping .255 or .0 (we avoid .0 by preincrimenting below)
          {
            g_net_scanidx=0;
            g_net_scanif++;
          }

          // wrap interfaces
          if (g_net_scanif >= sizeof(g_networks)/sizeof(g_networks[0]) || !g_networks[g_net_scanif].ip)
          {
            g_net_scanif=g_net_scanidx=0;
          }

          int maskbits=g_networks[g_net_scanif].maskbits;

          DWORD myip=ntohl(g_networks[g_net_scanif].ip);
          DWORD ip=(myip & (0xFFFFFFFF << (32-maskbits))) +  ++g_net_scanidx; // we preincriment here so that we don't try to ping the .0 etc
          DWORD the_ip=htonl(ip);
          g_last_scan_ip=ip;
       
          // don't show self
          if (the_ip != g_networks[g_net_scanif].ip) 
          {
            // ping the_ip
            ping_add(the_ip,g_config_pingtimeout);
          }
        }

        DWORD ip;
        int t=ping_run(&ip);
        if (ip)
        {
          ip=ntohl(ip);

          MonitoredHost m(ip,0);
          MonitoredHost *mm = &m;
          MonitoredHost **res;

          res = (MonitoredHost **)bsearch((void *) &mm,(void *)g_hostlist.GetList(),g_hostlist.GetSize(),sizeof(void *),HostSearchCompare);

          if (t < 0) // check to see if it is in the list, if it is, go towards removing it
          {
            if (res) // if in list
            {
              mm=*res;
              if (mm->m_failed_ping_count++ > 4) // if count of missed pings is > 5
              {
                // remove from list
                if (mm == m_scanning_host) m_scanning_host=0;

                delete mm;
                g_hostlist.Delete(res - g_hostlist.GetList());
                // notify list of redraw needed

                ListView_SetItemCount(g_listview,g_hostlist.GetSize());
                ListView_RedrawItems(g_listview,0,g_hostlist.GetSize()-1);
              }
            }
          }
          else
          {
            if (!res) // if not in list, add
            {
              mm=new MonitoredHost(ip,t);
          
              g_hostlist.Add(mm);
              qsort((void *)g_hostlist.GetList(),g_hostlist.GetSize(),sizeof(void *),HostSearchCompare);

              ListView_SetItemCount(g_listview,g_hostlist.GetSize());
              ListView_RedrawItems(g_listview,0,g_hostlist.GetSize()-1);
              if (0) {
                char buf[512];
                wsprintf(buf,"%d.%d.%d.%d",((ip>>24)&0xff),(ip>>16)&0xff,(ip>>8)&0xff,(ip)&0xff);
                MessageBox(hwndDlg,buf,"Found new IP",MB_OK);
              }
            }
            mm->m_failed_ping_count=0;
            mm->m_last_ping=t;

            if (res)
            {
              int idx=res-g_hostlist.GetList();
              ListView_RedrawItems(g_listview,idx,idx);
            }

            // update info

    //        char buf[512];
      //      wsprintf(buf,"%dms: %d.%d.%d.%d\n",t,((ip>>24)&0xff),(ip>>16)&0xff,(ip>>8)&0xff,(ip)&0xff);
        //    OutputDebugString(buf);
          }
        }
      }
      else if (wParam == 100)
      {
        DWORD now=GetTickCount();
        if (g_last_scan_ip) 
        {
          char buf[512];
          wsprintf(buf,"Scanning: %d.%d.%d.%d",(g_last_scan_ip>>24)&0xff,(g_last_scan_ip>>16)&0xff,(g_last_scan_ip>>8)&0xff,g_last_scan_ip&0xff);
          SetDlgItemText(hwndDlg,IDC_STATUS,buf);
        }

        if (!m_scanning_host) // we only do this once in a while (every second or so)
        {
          run_portscan(NULL); // make sure it knows to reset

          int x;
          for (x = 0; x < g_hostlist.GetSize(); x ++)
          {
            if (!g_hostlist.Get(x)->m_has_scanned && g_hostlist.Get(x)->m_address)
            {
              m_scanning_host=g_hostlist.Get(x);
              break;
            }
          }        
        }

        if (now > g_last_netrefreshtime + g_config_netrefresh || now < g_last_netrefreshtime - 2000000000)
        {
          if (getNetworkAdapterList(g_networks,sizeof(g_networks)/sizeof(g_networks[0])))
          {
            // update text
            WDL_String s;
            int x;
            int nc=0;
            s.Set("Networks: ");
            for (x = 0; x < sizeof(g_networks)/sizeof(g_networks[0]); x ++)
            {
              if (!g_networks[x].ip) break;
              char str[128];
              int ip=g_networks[x].ip;
              wsprintf(str,"%d.%d.%d.%d",(ip&0xff),(ip>>8)&0xff,(ip>>16)&0xff,(ip>>24)&0xff);
              wsprintf(str+strlen(str),"/%d",g_networks[x].maskbits);
              if (nc) s.Append(",");
              nc=1;             
              s.Append(str);             
            }
            SetDlgItemText(hwndDlg,IDC_NETWORKS,s.Get());
            g_net_scanif=g_net_scanidx=0;
            for (x = 0; x < g_hostlist.GetSize(); x++)
            {
              delete g_hostlist.Get(x);
            }
            g_hostlist.Empty();
            ping_clear();
            m_scanning_host=0;

            ListView_SetItemCount(g_listview,0);
            ListView_RedrawItems(g_listview,0,0);

          }
          g_last_netrefreshtime = now = GetTickCount();
        }
      }
    return 0;
  }
  return 0;
}